/*
    Qt Mapper - A GPS map application
    Copyright (C) 2008  Ixonos Plc. Authors:

        Antero Lehtonen - antero.lehtonen@ixonos.com
        Atte Tihinen - atte.tihinen@ixonos.com
        Jaakko Putaala - jaakko.putaala@ixonos.com
        Teppo Pennanen - teppo.pennanen@ixonos.com

    Qt Mapper is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    Qt Mapper is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Qt Mapper; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
    USA.
*/

#include "bar.h"

#include <QFontMetrics>
#include <QPainter>

//! Margins used to layout drawing
const int valueFontMargin = 10;
const int barMargin = 2;

//! Constructor, with parameters given as a QRect object
Bar::Bar(QRect &aRect)
{
    barRect.setTopLeft(aRect.topLeft());
    barRect.setBottomRight(aRect.bottomRight());

    value = 0;

    font.setFamily("Arial");

    // Calculating approx font metrics with example text.
    // This is close enough result to layout view nicely
    QFontMetrics fontMetrics(font);
    fontHeight = fontMetrics.height();
    QString valueString("00");
    textWidth = fontMetrics.width(valueString);

    measure() ;
}

//! Destructor
Bar::~Bar()
{
}

/*!
 * Measures bar by the given parameters. Always called when paintevent
 * is called.
 */
void Bar::measure()
{
	int textIndent = 0;
    if(textWidth < barRect.width())
    {
    	textIndent = (barRect.width() - textWidth)/2;
    }
    // Draw value above the bar
    valueTextPoints.setX(barRect.left() + textIndent);
    valueTextPoints.setY(barRect.bottom() - value - fontHeight - valueFontMargin * 2);

    // Drawn id below the bars
    idTextPoints.setX(barRect.left() + textIndent) ;
    idTextPoints.setY(barRect.bottom() - valueFontMargin);

    barRect.setTop( barRect.bottom() - value - fontHeight - valueFontMargin);

}

//! Sets new ID name for the bar
void Bar::setId(QString &aId)
{
    id = aId ;
}

//! return bar's ID as QString.
QString Bar::getId()
{
    return id ;
}

/*!
 * Called by paintevent. Draws bar and defines color and height for the bar
 * by value.
 */
void Bar::draw(QPainter * aPainter)
{
    measure() ;
    QLinearGradient rectColor;

    QPoint bl = barRect.bottomLeft();
	// Reduce space left for id text
    bl.setY( bl.y() - fontHeight - valueFontMargin);
    // Leave space between bars
    bl.setX( bl.x() + barMargin);

    QPoint tl = QPoint(bl.x(), bl.y() - value);


    rectColor.setStart(tl) ;
    rectColor.setFinalStop(bl.x(),barRect.bottom());
    if (id.length() > 1 && value > 0) {
        aPainter->setPen(outline) ;

        if (value <= 10) {
            rectColor.setColorAt(1.0, Qt::black);
            rectColor.setColorAt(0.0, Qt::red);
            aPainter->setBrush(rectColor) ;
        } else if ((value <= 30) && (value > 10)) {
            rectColor.setColorAt(1.0, Qt::black);
            rectColor.setColorAt(0.0, Qt::yellow);
            aPainter->setBrush(rectColor) ;
        } else if ((value > 30) && (value < 50)) {
            rectColor.setColorAt(1.0, Qt::black);
            rectColor.setColorAt(0.0, Qt::blue);
            aPainter->setBrush(rectColor) ;
        } else if (value >= 50) {
            rectColor.setColorAt(1.0, Qt::black);
            rectColor.setColorAt(0.0, Qt::green);
            aPainter->setBrush(rectColor) ;
        } else {
            rectColor.setColorAt(1.0, Qt::black);
            rectColor.setColorAt(0.0, Qt::black);
            aPainter->setBrush(rectColor) ;
        }

        QPoint br = barRect.bottomRight();
        br.setY( br.y() - fontHeight - valueFontMargin );
        br.setX( br.x() - barMargin );
        aPainter->drawRect(QRect(tl,br));

        aPainter->setFont(font) ;
        QString valueString ;
        valueString.setNum(value) ;
        aPainter->drawText(valueTextPoints, valueString) ;
        aPainter->drawText(idTextPoints, id) ;
    }
}

//! Updates new value for the bar and calculates right value for the bar.
void Bar::setSNRValue( int aValue )
{
	value = aValue;
}

//! returns current snr value
int Bar::getSNRValue( )
{
	return value;
}

//! Update bar rect
void Bar::setRect(const QRect& aRect)
{
  barRect = aRect;
}

//! Get bar rect
QRect& Bar::getRect()
{
  return barRect;
}
