/*
    Qt Mapper - A GPS map application
    Copyright (C) 2008  Ixonos Plc. Authors:

        Antero Lehtonen - antero.lehtonen@ixonos.com
        Atte Tihinen - atte.tihinen@ixonos.com
        Jaakko Putaala - jaakko.putaala@ixonos.com
        Teppo Pennanen - teppo.pennanen@ixonos.com

    Qt Mapper is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    Qt Mapper is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Qt Mapper; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
    USA.
*/

#include "diagrams.h"
#include "bar.h"
#include "plot.h"
#include "satelliteDrawing.h"

#include <QPaintEvent>
#include <QPoint>


//! Overloaded Constructor, takes parameters from the widget's geometry
Diagrams::Diagrams(int aCountOfBars)
{
    countOfBars = aCountOfBars;
    init();
    for (int i = 0; i < countOfBars; i++) {
        bars[i] = NULL;
        sat[i] = NULL;
    }
    createBars();
	layouted = false;
}

//! Variables initialized here to avoid repetition in constructors
void Diagrams::init()
{
    maxValue = 0;
    minValue = 0;

    // Header Texts.
    headerFont = QFont("Arial", 8);
    headerFont.setBold(true);
    headerFont.setUnderline(true);
}

//! Deletes all satelliteDrawing objects and Bar objects
Diagrams::~Diagrams()
{
    for (int i = 0; i < countOfBars; i++) {
        delete bars[i];
        delete sat[i];
    }
}

/*!
 * Takes measures from the QWidget's geometry and calculates new measures
 * for the drawing areas.
 */
void Diagrams::measure()
{

    int geometryHeight = geometry().height();
    int geometryWidth = geometry().width();
    int geometryX = geometry().x();
    int geometryY = geometry().y();

    QFontMetrics headerMetrics(headerFont);
    int headerHeight = headerMetrics.height();

	// Left header text position
    headerPlaceLeft = QPoint(0, headerHeight);
    // Right header text position
    headerPlaceRight = QPoint(geometryWidth / 2, headerHeight);

    // Using text height as margin
    int x = geometryX;
    int y = geometryY + headerHeight;
    int width = geometryWidth;
    int height = geometryHeight - headerHeight;

    // measure drawing areas.
    // Bar area (snr)
    drawAreaLeft.setX(x);
    drawAreaLeft.setY(y + headerHeight);
    drawAreaLeft.setBottom( y + height);
    drawAreaLeft.setWidth(width / 2);

	// satellelite locations area
    drawAreaRight.setX(x + width / 2 );
    drawAreaRight.setY(y + headerHeight );
    // Appears to be that satellite location drawer has some calculation problems,
    // thus not adding starting y for height to get some margin :)
    drawAreaRight.setBottom( height );
    drawAreaRight.setWidth( width / 2 );

    barAreaRect = drawAreaLeft;
}

//! Sets maximum value for the signal noise ratio
void Diagrams::setMaximumValue(int aMaxValue)
{
    maxValue = aMaxValue;
}

//! return maximum value
int Diagrams::getMaximumValue()
{
    return maxValue;
}

//! Sets minimum value for the signal noise ratio
void Diagrams::setMinimumValue(int aMinValue)
{
    minValue = aMinValue;
}

//! return minimum value
int Diagrams::getMinimumValue()
{
    return minValue;
}

//! Via this function new signal noise ratio is updated
void Diagrams::setSNRValue(int aIndex, int aValue)
{
    if (aValue > maxValue) {
        aValue = maxValue;
    }
    else if (aValue < minValue) {
        aValue = minValue;
    }

    if (aIndex < countOfBars || aIndex >= 0) {
        int snr = bars[aIndex]->getSNRValue( );
        if(snr != aValue)
        {
        	bars[aIndex]->setSNRValue( aValue );
        	repaint(drawAreaLeft);
        }
    }

}

//! return count of bars. Describes satellites count also.
int Diagrams::getCountOfBars()
{
    return countOfBars;
}

//! Creates bar and satellite objects. Called from all constructors.
void Diagrams::createBars()
{
    int xpos = 0;
    int barwidth = barAreaRect.width() / countOfBars;
    for (int i = 0; i < countOfBars;i++) {
    	QRect barRect = QRect( barAreaRect.left() + xpos, barAreaRect.top(), barwidth, barAreaRect.height() );
        xpos += barwidth;
        delete bars[i];
        bars[i] = new Bar(barRect);
        delete sat[i];
        sat[i] = new SatelliteDrawing(drawAreaRight);
    }
    plot = new Plot(drawAreaRight);
}

//! Updates layout for the bars
void Diagrams::layoutBars()
{
    int xpos = 0;
    int barwidth = barAreaRect.width() / countOfBars;
    for (int i = 0; i < countOfBars;i++) {
    	QRect barRect = QRect( barAreaRect.left() + xpos, barAreaRect.top(), barwidth, barAreaRect.height() );
        xpos += barwidth;
        if(bars[i] == NULL)
        {
        	bars[i] = new Bar(barRect);
        }
        else
        {
        	bars[i]->setRect(barRect);
        }
        if(sat[i] == NULL)
        {
        	sat[i] = new SatelliteDrawing(drawAreaRight);
        }
        else
        {
        	sat[i]->setRect(drawAreaRight);
        }
    }
    plot = new Plot(drawAreaRight);
}
/*!
 * This is virtual protected in parent class. Have to implement when
 * custom drawing is used.
 */
void Diagrams::paintEvent(QPaintEvent* aEvent)
{
    if (aEvent->type() == QEvent::Paint) {
    	if(!layouted)
    	{
			measure();
			layoutBars();
	        plot->updateArea(drawAreaRight);
	        for (int i = 0;i < countOfBars;i++) {
				// Plot has weirdly separated draw and control area rects
	        	sat[i]->updateArea(plot->getDrawArea());
	        }
			layouted = true;
    	}
        //left side of dialog
        QString headerLeft(tr("Satellites' SNRs"));
        QString headerRight(tr("Satellites' Locations"));

        painterLeft.begin(this);
        painterLeft.setFont(headerFont);
        painterLeft.drawText(headerPlaceLeft, headerLeft);

        for (int i = 0; i < countOfBars;i++) {
            bars[i]->draw(&painterLeft);
        }

        painterLeft.end();

        //right side of dialog
        painterRight.begin(this);

        plot->draw(&painterRight);
        painterRight.setFont(headerFont);
        painterRight.setPen(Qt::black);
        painterRight.drawText(headerPlaceRight, headerRight);

        for (int i = 0;i < countOfBars;i++) {
            sat[i]->draw(&painterRight);
        }

        painterRight.end();
    }
}

//! Sets elevation value for the wanted satellite drawing.
void Diagrams::setElevation(int aIndex, int aElevation)
{
    if (aIndex < countOfBars || aIndex >= 0) {
        int ele = sat[aIndex]->getElevation( );
        if(ele != aElevation)
        {
        	sat[aIndex]->setElevation( aElevation );
        	repaint(drawAreaRight);
        }
    }
}

//! Sets azimuth value for the wanted satellite drawing.
void Diagrams::setAzimuth(int aIndex, int aAzimuth)
{
    if (aIndex < countOfBars || aIndex >= 0) {
        int azi = sat[aIndex]->getAzimuth( );
        if(azi != aAzimuth)
        {
        	sat[aIndex]->setAzimuth( aAzimuth );
        	repaint(drawAreaRight);
        }
    }
}

//! Replace Id
void Diagrams::setId(int aIndex, QString &aId)
{
    bars[aIndex]->setId(aId);
    sat[aIndex]->setId(aId);
}

//! return Satellite's ID number as QString
QString Diagrams::getId(int aIndex)
{
    return bars[aIndex]->getId();
}
