/*
    Qt Mapper - A GPS map application
    Copyright (C) 2008  Ixonos Plc. Authors:

        Antero Lehtonen - antero.lehtonen@ixonos.com
        Atte Tihinen - atte.tihinen@ixonos.com
        Jaakko Putaala - jaakko.putaala@ixonos.com
        Teppo Pennanen - teppo.pennanen@ixonos.com

    Qt Mapper is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    Qt Mapper is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Qt Mapper; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
    USA.
*/

#include <stdio.h>
#include <QString>
#include "gpsdclient.h"

// #include "gps.h"    /* for reading GPS data from daemon */

//! Constructor
GpsdClient::GpsdClient()
{
    socket = NULL;
}

//! Destructor
GpsdClient::~GpsdClient()
{
    stopGps();
}

//! Starts gps when called.
void GpsdClient::startGps()
{
    restartGps();
}

//! When starting or trying to start gps this function is called.
//! A callback function for gps starter, that is supposed to start gpsd if needed
//! gpsd is needed to be running in order to be enabled to connect that here :)
void GpsdClient::gpsRunning(bool run, QString error)
{
    const QString localAddress("127.0.0.1");

    if (run) {

        if(socket)
        {
        	socket->abort();
        }
        if(!socket)
        {
        	socket = new QTcpSocket(this);

			connect(socket, SIGNAL(error(QAbstractSocket::SocketError)),
					this, SLOT(showError(QAbstractSocket::SocketError)));
			connect(socket, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
					this, SLOT(readState(QAbstractSocket::SocketState)));
			connect(socket, SIGNAL(connected()), this, SLOT(socketConnected()));
			connect(socket, SIGNAL(readyRead()), this, SLOT(readData()));
        }
        socket->connectToHost(localAddress, static_cast<uint>(gpsdDefaultPort),
                              QIODevice::ReadWrite);
    }
    else {
        emit errorOccured(error);
    }
}

//! When some error in socket connection occurs this function is called.
void GpsdClient::showError(QAbstractSocket::SocketError socketError)
{
    emit errorOccured(tr("Socket error: %1").arg(socketError));
}

//! When socket is connected this is called.
void GpsdClient::socketConnected()
{
    const char *lineBreak = "r+\r\n";

    socket->write(lineBreak, sizeof(lineBreak));
}

//! Reads data from the gps socket
void GpsdClient::readData()
{
    const QString newLineCharacter("\n");
    QString unparsedNmeaString(socket->readAll());
    QString parsedNmeaString;
    int count = unparsedNmeaString.count();
    QRegExp delimiter(newLineCharacter);

    for (int i = 0; i < count; i++) {
        parsedNmeaString.append(unparsedNmeaString.section(delimiter, 0, 0));
        parsedNmeaString.append(newLineCharacter);

        if (parsedNmeaString.length() > 1) {
            emit nmeaSentenceReady(parsedNmeaString);
        }

        unparsedNmeaString.remove(0, unparsedNmeaString
                                  .indexOf(delimiter) + 1);
        parsedNmeaString.clear();
    }
}

//! This function is always called when sockets' state is changed
void GpsdClient::readState(QAbstractSocket::SocketState socketState)
{
    Q_UNUSED(socketState);
}

//! restarts gps when called. Can also be used for starting the gps
void GpsdClient::restartGps()
{
    stopGps();
    gpsRunning(true,"");
}

//! stops gps when called.
void GpsdClient::stopGps()
{
	if(socket)
	{
		socket->abort();
	}
	delete socket;
	socket = NULL;

}

