/*
    Qt Mapper - A GPS map application
    Copyright (C) 2008  Ixonos Plc. Authors:

        Antero Lehtonen - antero.lehtonen@ixonos.com
        Atte Tihinen - atte.tihinen@ixonos.com
        Jaakko Putaala - jaakko.putaala@ixonos.com
        Teppo Pennanen - teppo.pennanen@ixonos.com

    Qt Mapper is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    Qt Mapper is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Qt Mapper; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
    USA.
*/

#include <QDebug>
#include <QPainter>
#include "satelliteDrawing.h"
#include "unitconverter.h"
#include "math.h"
const int fontSize = 10;
//! Constructor, parameters inside a QRect
SatelliteDrawing::SatelliteDrawing(QRect &aRect)
{
	controlRect = new QRect();
	modelRect = new QRect();
    setRect(aRect);

    font.setPixelSize(fontSize);
    // Calculating approx font metrics with example text.
    // This is close enough result to layout view nicely
    QFontMetrics fontMetrics(font);
    int fontHeight = fontMetrics.height();
    QString valueString("00");
    int textWidth = fontMetrics.width(valueString);
    if(fontHeight > textWidth)
    {
        satelliteDotSize = fontHeight;
    }
    else
    {
        satelliteDotSize = textWidth;
    }
    init() ;
}

//! Destructor
SatelliteDrawing::~SatelliteDrawing()
{
	delete controlRect;
	delete modelRect;
}

/*!
 * This function initializes all variables and is called from both constructors.
 */
void SatelliteDrawing::init()
{
    idPen = QPen(Qt::black);
    elevation = 0;
    azimuth = 0;
    modelStyle = QBrush(Qt::gray);
}

//! Sets new Id for the satellite.
void SatelliteDrawing::setId(QString &aId)
{
    id = aId;
}

//! Sets elevation value for the satellite
void SatelliteDrawing::setElevation(int aElevation)
{
    elevation = aElevation;
}

//! Gets elevation value for the satellite
int SatelliteDrawing::getElevation()
{
    return elevation;
}

//! Sets azimuth value for the satellite
void SatelliteDrawing::setAzimuth(int aAzimuth)
{
    azimuth = aAzimuth;
}

//! Gets azimuth value for the satellite
int SatelliteDrawing::getAzimuth()
{
    return azimuth;
}

//! Updates drawing area
void SatelliteDrawing::updateArea(QRect &aRect)
{
	setRect(aRect);
}

/*!
 * Calculates satellites position on the plot by the given azimuth and
 * elevation values.
 */
void SatelliteDrawing::calculatePosition()
{
    UnitConverter *uc = new UnitConverter();
    int tmp = (int)(uc->deg2rad(azimuth));
    modelRect->setX((int)(controlRect->x() + controlRect->width() / 2 +
                   (90 - elevation) * (controlRect->width() / 2) / 90 * sinf(tmp)));
    modelRect->setY((int)(controlRect->y() + controlRect->height() / 2 -
                   (90 - elevation) * (controlRect->height() / 2) / 90 * cosf(tmp)));
    modelRect->setX(modelRect->x());
    modelRect->setY(modelRect->y());
    modelRect->setHeight(satelliteDotSize);
    modelRect->setWidth(satelliteDotSize);
    delete uc;
}

//! Called by paintevent. Draws the circle describing the satellite and it's ID
void SatelliteDrawing::draw(QPainter *aPainter)
{
    if (azimuth >= 0 && elevation >= 0 && id.length() > 1) {
        calculatePosition();
        aPainter->setPen(Qt::gray);
        aPainter->setBrush(modelStyle);
        aPainter->drawEllipse(*modelRect);
        idPen.setColor(Qt::black);
        aPainter->setPen(idPen);
        aPainter->setFont(font);
        aPainter->drawText(*modelRect, id);
    }
}

//! Update rect
void SatelliteDrawing::setRect(const QRect& aRect)
{
	controlRect->setRect(aRect.x(), aRect.y(), aRect.width(), aRect.height());
}
